@@ -3,10 +3,10 @@ from hypernetx.exception import (
     HyperNetXError,
     HyperNetXNotImplementedError,
 )
-from hypernetx.read_write import to_pickle, load_from_pickle
 from hypernetx.classes import *
 from hypernetx.reports import *
 from hypernetx.drawing import *
 from hypernetx.algorithms import *
+from hypernetx.algorithms.contagion import *
 from hypernetx.utils import *
 from hypernetx.utils.toys import *
